﻿using Acme.BookStore.Domain;
using Microsoft.EntityFrameworkCore;
using Microsoft.EntityFrameworkCore.Metadata;
using Microsoft.EntityFrameworkCore.Storage.ValueConversion;
using System;
using Volo.Abp.AuditLogging.EntityFrameworkCore;
using Volo.Abp.BackgroundJobs.EntityFrameworkCore;
using Volo.Abp.Data;
using Volo.Abp.DependencyInjection;
using Volo.Abp.EntityFrameworkCore;
using Volo.Abp.FeatureManagement.EntityFrameworkCore;
using Volo.Abp.Identity;
using Volo.Abp.Identity.EntityFrameworkCore;
using Volo.Abp.OpenIddict.EntityFrameworkCore;
using Volo.Abp.PermissionManagement.EntityFrameworkCore;
using Volo.Abp.SettingManagement.EntityFrameworkCore;
using Volo.Abp.TenantManagement;
using Volo.Abp.TenantManagement.EntityFrameworkCore;

namespace Acme.BookStore.EntityFrameworkCore;

[ReplaceDbContext(typeof(IIdentityDbContext))]
[ReplaceDbContext(typeof(ITenantManagementDbContext))]
[ConnectionStringName("Default")]
public class BookStoreDbContext :
    AbpDbContext<BookStoreDbContext>,
    IIdentityDbContext,
    ITenantManagementDbContext
{
    /* Add DbSet properties for your Aggregate Roots / Entities here. */

    #region Entities from the modules

    /* Notice: We only implemented IIdentityDbContext and ITenantManagementDbContext
     * and replaced them for this DbContext. This allows you to perform JOIN
     * queries for the entities of these modules over the repositories easily. You
     * typically don't need that for other modules. But, if you need, you can
     * implement the DbContext interface of the needed module and use ReplaceDbContext
     * attribute just like IIdentityDbContext and ITenantManagementDbContext.
     *
     * More info: Replacing a DbContext of a module ensures that the related module
     * uses this DbContext on runtime. Otherwise, it will use its own DbContext class.
     */

    //Identity
    public DbSet<IdentityUser> Users { get; set; }
    public DbSet<IdentityRole> Roles { get; set; }
    public DbSet<IdentityClaimType> ClaimTypes { get; set; }
    public DbSet<OrganizationUnit> OrganizationUnits { get; set; }
    public DbSet<IdentitySecurityLog> SecurityLogs { get; set; }
    public DbSet<IdentityLinkUser> LinkUsers { get; set; }
    public DbSet<IdentityUserDelegation> UserDelegations { get; set; }

    public DbSet<SizeChart> SizeChart { get; set; }

    public DbSet<ProductGender> ProductGender { get; set; }
    public DbSet<ProductType> ProductType { get; set; }

    // Tenant Management
    public DbSet<Tenant> Tenants { get; set; }
    public DbSet<TenantConnectionString> TenantConnectionStrings { get; set; }

    #endregion

    public BookStoreDbContext(DbContextOptions<BookStoreDbContext> options)
        : base(options)
    {

    }

    protected override void OnModelCreating(ModelBuilder builder)
    {
        base.OnModelCreating(builder);

        /* Include modules to your migration db context */

        builder.ConfigurePermissionManagement();
        builder.ConfigureSettingManagement();
        builder.ConfigureBackgroundJobs();
        builder.ConfigureAuditLogging();
        builder.ConfigureIdentity();
        builder.ConfigureOpenIddict();
        builder.ConfigureFeatureManagement();
        builder.ConfigureTenantManagement();

        /* Configure your own tables/entities inside here */

        //builder.Entity<YourEntity>(b =>
        //{
        //    b.ToTable(BookStoreConsts.DbTablePrefix + "YourEntities", BookStoreConsts.DbSchema);
        //    b.ConfigureByConvention(); //auto configure for the base class props
        //    //...
        //});

        builder.Entity<ProductGender>(b =>
        {
            b.ToTable(BookStoreConsts.DbTablePrefix + "product_gender", BookStoreConsts.DbSchema);
            b.Property(uc => uc.Name).HasMaxLength(256).IsRequired();
        });


        builder.Entity<ProductType>(b =>
        {
            b.ToTable(BookStoreConsts.DbTablePrefix + "product_type", BookStoreConsts.DbSchema);
            b.Property(uc => uc.Name).HasMaxLength(256).IsRequired();
        });

        builder.Entity<SizeChart>(b =>
        {
            b.ToTable(BookStoreConsts.DbTablePrefix + "size_chart", BookStoreConsts.DbSchema);

            b.Property(r => r.BrandType).HasMaxLength(32).HasConversion(new EnumToStringConverter<Acme.BookStore.Domain.BrandType>());
            b.HasOne(u => u.ProductGender).WithMany().HasForeignKey(x => x.ProductGenderId);
            b.HasOne(u => u.ProductType).WithMany().HasForeignKey(x => x.ProductTypeId);

            b.Property(uc => uc.OriginalSize).HasMaxLength(512).IsRequired();
            b.Property(uc => uc.USSize).HasMaxLength(512).IsRequired(false);
            b.Property(uc => uc.CanadaSize).HasMaxLength(512).IsRequired(false);
            b.Property(uc => uc.ChinaSize).HasMaxLength(512).IsRequired(false);
            b.Property(uc => uc.SPSize).HasMaxLength(512).IsRequired(false);
            b.Property(uc => uc.RUSize).HasMaxLength(512).IsRequired(false);
            b.Property(uc => uc.UKSize).HasMaxLength(512).IsRequired(false);
            b.Property(uc => uc.EUSize).HasMaxLength(512).IsRequired(false);
            b.Property(uc => uc.JPSize).HasMaxLength(512).IsRequired(false);
            b.Property(uc => uc.ITSize).HasMaxLength(512).IsRequired(false);
            b.Property(uc => uc.FRSize).HasMaxLength(512).IsRequired(false);
            b.Property(uc => uc.S1UsSize).HasMaxLength(512).IsRequired(false);
            b.Property(uc => uc.S2CanSize).HasMaxLength(512).IsRequired(false);
            b.Property(uc => uc.S3UkSize).HasMaxLength(512).IsRequired(false);
            b.Property(uc => uc.S4EuSize).HasMaxLength(512).IsRequired(false);
            b.Property(uc => uc.S5JpSize).HasMaxLength(512).IsRequired(false);
            b.Property(uc => uc.S6).HasMaxLength(512).IsRequired(false);
            b.Property(uc => uc.S7UsSize).HasMaxLength(512).IsRequired(false);
            b.Property(uc => uc.S8CanSize).HasMaxLength(512).IsRequired(false);
            b.Property(uc => uc.S9UkSize).HasMaxLength(512).IsRequired(false);
            b.Property(uc => uc.S10EuSize).HasMaxLength(512).IsRequired(false);
            b.Property(uc => uc.S11JpSize).HasMaxLength(512).IsRequired(false);
            b.Property(uc => uc.S12ItSize).HasMaxLength(512).IsRequired(false);
            b.Property(uc => uc.S13FrSize).HasMaxLength(512).IsRequired(false);
            b.Property(uc => uc.S14ItSize).HasMaxLength(512).IsRequired(false);
            b.Property(uc => uc.S15FrSize).HasMaxLength(512).IsRequired(false);
            b.Property(uc => uc.CNSize).HasMaxLength(512).IsRequired(false);
            b.Property(uc => uc.MXSPSize).HasMaxLength(512).IsRequired(false);
            b.Property(uc => uc.UYSize).HasMaxLength(512).IsRequired(false);
            b.Property(uc => uc.IDSize).HasMaxLength(512).IsRequired(false);
            b.Property(uc => uc.KRSize).HasMaxLength(512).IsRequired(false);
            b.Property(uc => uc.Online).HasMaxLength(512).IsRequired(false);

        });

        builder.RewriteName();
    }

  
}

public static class A
{
    //https://github.com/efcore/EFCore.NamingConventions
    public static void RewriteName(this ModelBuilder builder)
    {
        foreach (var entity in builder.Model.GetEntityTypes())
        {
            var tableName = entity.GetTableName();

            if (!tableName.Contains(BookStoreConsts.DbTablePrefix))
            {
                continue;
            }

            foreach (var property in entity.GetProperties())
            {
                var baseColumnName = StoreObjectIdentifier.Create(property.DeclaringType, StoreObjectType.Table) is { } tableIdentifier
           ? property.GetDefaultColumnName(tableIdentifier)
           : property.GetDefaultColumnName();
                property.SetColumnName(baseColumnName.ToSnakeCase());

                foreach (var key in entity.GetKeys())
                {
                    key.SetName(key.GetName().ToSnakeCase());
                }

                foreach (var key in entity.GetForeignKeys())
                {
                    key.SetConstraintName(key.GetConstraintName().ToSnakeCase());
                }

                foreach (var index in entity.GetIndexes())
                {
                    //index.SetName(index.GetName().ToSnakeCase());

                    index.SetDatabaseName(index.GetDatabaseName().ToSnakeCase());
                }
            }
        }
    }
}
